#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    Setup file for pref_vote.

    This file was generated with PyScaffold 2.5.8, a tool that easily
    puts up a scaffold for your new Python project. Learn more under:
    http://pyscaffold.readthedocs.org/
"""

from setuptools import setup


setup(name='pref_vote',
      description='Preferential voting tool',
      version='0.0.4',
      url='https://github.com/jmhossler/pref_vote',
      author='John Hossler',
      author_email='john.m.hossler@gmail.com',
)
